'use strict';

import { createElement, Component, PropTypes, findDOMNode } from 'rax';
import View from 'nuke-view';
import Env from 'nuke-env';
import { connectStyle } from 'nuke-theme-provider';
import SwipeEvent from 'nuke-swipe';
import stylesProvider from '../styles';

const { appInfo } = Env;
const isIOS = appInfo.platform.toLowerCase() === 'ios';
/**
 * Range
 * @description  区间滑块
 */
class Range extends Component {
  constructor(props) {
    super(props);

    this.webStart = this.webStart.bind(this);
    this.webMove = this.webMove.bind(this);
    this.webMove2 = this.webMove2.bind(this);
    this.renderHandler = this.renderHandler.bind(this);
    this.state = {
      start: 0,
      end: 0,
      active: false,
      value: props.value || props.defaultValue || 0,
      barWidth: 0,
      timeout: 100,
      left: 0,
      selectRange: [0, 0],
      minDiffX: 0,
    };
    this.selectRange = [0, 0];
    this.dpr = 2;
    this.ep = false;
  }

  componentDidMount() {
    this.block1 = findDOMNode(this.refs['slide-block-1']); // 左侧滑块
    this.block2 = findDOMNode(this.refs['slide-block-2']); // 右侧滑块
    this.valueBar = findDOMNode(this.refs['slide-value-bar']); // 黄色值条
    this.barContainer = findDOMNode(this.refs['slide-bar-container']); // 滚动条容器

    this.initBaseEnv(); // ep设置，滑块位置
  }
  // 根据value和length计算x方向偏移值
  getDistanceByValue(value) {
    return (
      ((value - this.props.min) / (this.props.max - this.props.min)) *
      this.length
    );
  }

  getValueByDistance(diffX) {
    return Math.round(
      (diffX / this.length) * (this.props.max - this.props.min) + this.props.min
    );
  }

  initBaseEnv() {
    const { slider, highPerformance, min, max, themeStyle, style } = this.props;
    this.length = parseInt(style.width || themeStyle.container.width, 10); // 可滑动的距离，未计算padding
    this.leftDiffX = 0;
    const newData = { selectRange: this.state.selectRange };
    if (!slider || slider === 'single') {
      newData.start = this.getDistanceByValue(this.state.value);
    } else if (slider === 'double') {
      newData.start = this.getDistanceByValue(this.state.value[0]);
      newData.end = this.getDistanceByValue(this.state.value[1]);
      newData.barWidth = newData.end - newData.start;
      this.selectRange = this.state.value || this.state.defaultValue;
      newData.minDiffX = (this.state.minDiffX / (max - min)) * this.length;
    } else {
      throw new Error('unsupport slider type,use single or double replace');
    }
    if (appInfo.platform === 'android') {
      newData.timeout = 250;
    }
    this.setState({
      ...newData,
    });
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.value != this.props.value && !this.paning) {
      this.setState(
        {
          value: nextProps.value,
        },
        () => {
          this.initBaseEnv();
        }
      );
    }
  }

  computeContainerStyle() {
    const { handlerStyle, style, themeStyle } = this.props;
    const handlerWidth = handlerStyle.width
      ? handlerStyle.width
      : themeStyle.slideBlock.width;
    const handlerHeight = handlerStyle.height
      ? handlerStyle.height
      : themeStyle.slideBlock.height;
    return {
      ...style,
      width: this.length + handlerWidth,
      height: handlerHeight,
    };
  }

  computRangeBarStyle() {
    const { afterBarStyle, themeStyle } = this.props;
    return {
      ...themeStyle.bar,

      ...themeStyle.invalid,
      ...afterBarStyle,
      width: this.length,
    };
  }

  computValueBarStyle() {
    let left = 0;
    let width = 0;
    const { slider, disabled, barStyle, themeStyle } = this.props;
    if (!slider || slider === 'single') {
      width = this.state.start;
    } else {
      left = this.state.start;
      width = this.state.end - this.state.start;
    }
    return Object.assign(
      {},
      themeStyle.bar,
      {
        width,
        transform: `translateX(${left / this.dpr}px)`,
      },

      disabled ? themeStyle.disabled : themeStyle.valid,
      barStyle
    );
  }

  computBlock1Style() {
    return { transform: `translateX(${this.state.start / this.dpr}px)` };
  }

  computBlock2Style() {
    return { transform: `translateX(${this.state.end / this.dpr}px)` };
  }

  renderHandler() {
    const { handlerStyle, style, slider, themeStyle } = this.props;
    // native handle
    const blockEle1Style = Object.assign(
      {},
      themeStyle.slideBlock,
      isIOS ? themeStyle.slideBlockIOS : {},
      handlerStyle,
      this.computBlock1Style()
    );
    const blockEle2Style = Object.assign(
      {},
      themeStyle.slideBlock,
      isIOS ? themeStyle.slideBlockIOS : {},
      handlerStyle,
      this.computBlock2Style()
    );
    // h5 handle
    const webBlockEle1Style = {
      ...blockEle1Style,
      positon: 'absolute',
    };
    const webBlockEle2Style = {
      ...blockEle2Style,
      positon: 'absolute',
    };

    const webBlockEle1 = (
      <SwipeEvent
        ref="slide-block-1"
        onSwipeBegin={this.webStart}
        onSwipe={this.webMove}
        onEnd={this.webEnd}
        horizontalThreshold={10}
        initialVelocityThreshold={0.01}
        vertical={false}
        handlerStyle={webBlockEle1Style}
      />
    );
    const webBlockEle2 = (
      <SwipeEvent
        ref="slide-block-2"
        onSwipeBegin={this.webStart}
        onSwipe={this.webMove2}
        onSwipeEnd={this.webEnd}
        horizontalThreshold={10}
        initialVelocityThreshold={0.01}
        handlerStyle={webBlockEle2Style}
      />
    );

    if (slider !== 'double') {
      return webBlockEle1;
    }
    return [webBlockEle1, webBlockEle2];
  }

  render() {
    const {
      style,
      disabled,
      slider,
      handlerStyle,
      themeStyle,
      ...others
    } = this.props;

    return (
      <View
        ref="slide-bar-container"
        style={[themeStyle.container, this.computeContainerStyle()]}
      >
        <View style={[themeStyle.range, this.computRangeBarStyle()]}>
          <View ref="slide-value-bar" style={[this.computValueBarStyle()]}>
            <View ref="slide-value-bar-2" />
          </View>
        </View>
        {this.renderHandler()}
      </View>
    );
  }

  webStart(e) {
    this.webDiffX = e.distance;
  }
  webMove(e) {
    const distance = e.distance - this.webDiffX;
    this.webDiffX = e.distance;
    const start = this.state.start + distance * this.dpr;
    if (this.props.slider !== 'double') {
      if (start > this.length || start < 0) return;
    } else if (start > this.state.end - this.state.minDiffX || start < 0) {
      return;
    }
    this.setState({
      start,
    });
    this.props.onProcess &&
      this.props.onProcess(this.getValueByDistance(start));
  }
  webMove2(e) {
    const distance = e.distance - this.webDiffX;
    this.webDiffX = e.distance;
    const end = this.state.end + distance * this.dpr;
    if (this.props.slider !== 'double') {
      if (end > this.length || end < 0) return;
    } else if (end > this.length || end < this.state.start) {
      return;
    }
    this.setState({
      end,
    });
    this.props.onProcess &&
      this.props.onProcess([
        this.getValueByDistance(this.state.start),
        this.getValueByDistance(end),
      ]);
  }
  webEnd() {}
}
Range.PropTypes = {
  slider: PropTypes.string,
  defaultValue: PropTypes.any,
  value: PropTypes.any,
  min: PropTypes.number,
  max: PropTypes.number,
  disabled: PropTypes.bool,
  highPerformance: PropTypes.bool,
  onChange: PropTypes.func,
  onProcess: PropTypes.func,
  style: PropTypes.obj,
  handlerStyle: PropTypes.obj,
  barStyle: PropTypes.obj,
  afterBarStyle: PropTypes.obj,
};
Range.defaultProps = {
  slider: 'single',
  defaultValue: 0,
  min: 0,
  max: 100,
  disabled: false,
  highPerformance: true,
  handlerStyle: {},
  barStyle: {},
  afterBarStyle: {},
};
Range.displayName = 'Range';

const StyledRange = connectStyle(stylesProvider, { withRef: true })(Range);

export default StyledRange;
